==> xlr-devops-as-code-module-23.3.0-606.831-sources.jar: com/xebialabs/xlrelease/ascode/rest/XLRAsCodeResource.scala <==
package com.xebialabs.xlrelease.ascode.rest

import com.xebialabs.ascode.annotation.ExceptionToAsCodeResponse
import com.xebialabs.ascode.exception.AsCodeException
import com.xebialabs.ascode.yaml.model.Definition
import com.xebialabs.ascode.yaml.writer.DefinitionWriter.WriterConfig
import com.xebialabs.deployit.booter.local.utils.Strings
import com.xebialabs.deployit.core.rest.resteasy.Workdir
import com.xebialabs.deployit.core.rest.resteasy.Workdir.Clean.ONLY_ON_EXCEPTION
import com.xebialabs.deployit.security.permission.PlatformPermissions.ADMIN
import com.xebialabs.xlplatform.coc.dto.SCMTraceabilityData
import com.xebialabs.xlrelease.ascode.schema.JsonSchemaGenerator
import com.xebialabs.xlrelease.ascode.service.GenerateService.{CisConfig, GeneratorConfig}
import com.xebialabs.xlrelease.ascode.service._
import com.xebialabs.xlrelease.ascode.service.previewhandler.PreviewService
import com.xebialabs.xlrelease.ascode.utils.StaticVariables
import com.xebialabs.xlrelease.repository.Ids
import com.xebialabs.xlrelease.repository.Ids.SEPARATOR
import com.xebialabs.xlrelease.security.PermissionChecker
import grizzled.slf4j.Logging
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Controller

import java.io.InputStream
import javax.ws.rs._
import javax.ws.rs.core.Response

@Controller
@ExceptionToAsCodeResponse
@Path("/devops-as-code")
class XLRAsCodeResource @Autowired()(creationService: CreationService,
                                     generateService: GenerateService,
                                     previewService: PreviewService,
                                     jsonGenerator: JsonSchemaGenerator,
                                     permissionChecker: PermissionChecker) extends Logging {
  @POST
  @Produces(Array("application/json"))
  @Path("/apply")
  def createPipeline(definition: Definition, scmData: SCMTraceabilityData): Response = {
    try {
      val response = creationService.processBlueprint(definition, scmData)
      Response.ok(response).build()
    } catch {
      case e: Throwable =>
        logger.error(e.getMessage, e)
        throw e
    }
  }

  @POST
  @Path("/apply")
  @Consumes(Array("application/zip"))
  @Produces(Array("application/json"))
  @Workdir(prefix = StaticVariables.AS_CODE_GENERATE_PREFIX, clean = ONLY_ON_EXCEPTION)
  def interpretZip(in: InputStream, scmData: SCMTraceabilityData): Response = {
    val response = creationService.processArchive(in, scmData)
    Response.ok(response).build()
  }


  //noinspection ScalaStyle
  @GET
  @Produces(Array("application/zip", "text/vnd.yaml"))
  @Workdir(prefix = StaticVariables.AS_CODE_GENERATE_PREFIX, clean = ONLY_ON_EXCEPTION)
  @Path("/generate")
  def generateCi(@QueryParam("path") path: String,
                 @QueryParam("ciName") ciName: String,
                 @QueryParam("ciIds") ciIds: Array[String],
                 @DefaultValue("false") @QueryParam("defaults") defaults: Boolean,
                 @DefaultValue("false") @QueryParam("permissions") permissions: Boolean,
                 @DefaultValue("false") @QueryParam("users") users: Boolean,
                 @DefaultValue("false") @QueryParam("roles") roles: Boolean,
                 @DefaultValue("false") @QueryParam("environments") environments: Boolean,
                 @DefaultValue("false") @QueryParam("applications") applications: Boolean,
                 @DefaultValue("false") @QueryParam("secrets") includeSecrets: Boolean,
                 @DefaultValue("false") @QueryParam("templates") templates: Boolean,
                 @DefaultValue("false") @QueryParam("workflows") workflows: Boolean,
                 @DefaultValue("false") @QueryParam("triggers") triggers: Boolean,
                 @DefaultValue("false") @QueryParam("dashboards") dashboards: Boolean,
                 @DefaultValue("false") @QueryParam("configurations") configurations: Boolean,
                 @DefaultValue("false") @QueryParam("settings") settings: Boolean,
                 @DefaultValue("false") @QueryParam("riskProfiles") riskProfiles: Boolean,
                 @DefaultValue("false") @QueryParam("deliveryPatterns") deliveryPatterns: Boolean,
                 @DefaultValue("false") @QueryParam("variables") variables: Boolean,
                 @DefaultValue("false") @QueryParam("notifications") notificationSettings: Boolean,
                 @DefaultValue("false") @QueryParam("calendar") calendar: Boolean,
                 @DefaultValue("true") @QueryParam("includeSubFolders") includeSubFolders: Boolean,
                 @DefaultValue("false") @QueryParam("folderVersioning") folderVersioning: Boolean): Response = {

    // Only templates support generate by id
    if (!ciIds.forall(Ids.isReleaseId)) {
      throw new AsCodeException("Only template ids can be specified")
    }

    val searchScope = if (!ciIds.isEmpty) {
      ExactSearch(ciIds.toList)
    } else if (Strings.isNotBlank(path)) {
      if (path == SEPARATOR) {
        GlobalSearch
      } else {
        FolderSearch(path, null, includeSubFolders)
      }
    } else {
      AllSearch
    }

    val cisConfig = normalizeCisConfig(searchScope,
      CisConfig(
        generatePermissions = permissions,
        generateRoles = roles,
        generateUsers = users,
        generateEnvironments = environments,
        generateApplications = applications,
        includeSecrets = includeSecrets,
        generateTemplates = templates,
        generateWorkflows = workflows,
        generateTriggers = triggers,
        generateDeliveryPatterns = deliveryPatterns,
        generateDashboards = dashboards,
        generateConfigurations = configurations,
        generateSettings = settings,
        generateRiskProfiles = riskProfiles,
        generateVariables = variables,
        generateNotificationSettings = notificationSettings,
        generateCalendar = calendar,
        generateFolderVersioning = folderVersioning
      )
    )

    try {
      val generatorConfig = GeneratorConfig(Option(ciName).filter(_.nonEmpty), searchScope, cisConfig, permissionChecker.hasGlobalPermission(ADMIN))
      val definitions = generateService.generate(generatorConfig)
      Response.ok(WriterConfig(definitions, includeSecrets, writeDefaults = defaults)).build()
    } catch {
      case e: Throwable =>
        logger.error(e.getMessage, e)
        throw e
    }
  }

  @POST
  @Path("/preview")
  @Workdir(prefix = StaticVariables.AS_CODE_GENERATE_PREFIX, clean = ONLY_ON_EXCEPTION)
  @Produces(Array("application/json"))
  def preview(definition: Definition): Response = {
    val response = previewService.preview(definition)
    Response.ok(response).build()
  }

  @GET
  @Path("/schema")
  @Produces(Array("application/json"))
  def schema(): Response = {
    Response.ok(jsonGenerator.YamlSchema).build()
  }

  private def normalizeFolderScope(scope: SearchScope, clientValue: Boolean, folderScopeDefault: Boolean): Boolean = {
    scope match {
      case AllSearch => clientValue
      case GlobalSearch => false
      case _: FolderSearch => clientValue | folderScopeDefault
      case _: ExactSearch => false
    }
  }

  private def normalizeGlobalScope(scope: SearchScope, clientValue: Boolean): Boolean = {
    scope match {
      case AllSearch => clientValue
      case GlobalSearch => clientValue
      case _: FolderSearch => false
      case _: ExactSearch => false
    }
  }

  private def normalizeAnyScope(scope: SearchScope, clientValue: Boolean, folderScopeDefault: Boolean): Boolean = {
    scope match {
      case AllSearch => clientValue
      case GlobalSearch => clientValue
      case _: FolderSearch => clientValue | folderScopeDefault
      case _: ExactSearch => false
    }
  }

  private def validateScopeAndInput(scope: SearchScope, clientValues: CisConfig): Unit = {
    scope match {
      case GlobalSearch if !clientValues.hasGlobalTypes() && clientValues.hasFolderTypes() =>
        throw new AsCodeException("Could not generate a definition for path '/' when only folder-scoped types are requested")
      case FolderSearch(path, _, _) if clientValues.hasGlobalTypes() && !clientValues.hasFolderTypes() =>
        throw new AsCodeException(s"Could not generate a definition for folder '$path' when only global-scoped types are requested")
      case AllSearch if clientValues.isEmpty() =>
        throw new AsCodeException("Could not generate a definition without a path or some types specified")
      case _ =>
    }
  }

  private def normalizeCisConfig(scope: SearchScope, clientValues: CisConfig): CisConfig = {
    validateScopeAndInput(scope, clientValues)

    // When folder scope is needed but all folder types excluded it should default to all folder types
    val needAllFolderTypes = scope match {
      case _: FolderSearch => !clientValues.hasFolderTypes()
      case _ => false
    }

    val needTemplates = scope.isInstanceOf[ExactSearch]

    // Normalize so all values are only enabled when they are relevant to the scope being generated
    CisConfig(
      generatePermissions = normalizeAnyScope(scope, clientValues.generatePermissions, needAllFolderTypes),
      generateRoles = normalizeGlobalScope(scope, clientValues.generateRoles),
      generateUsers = normalizeGlobalScope(scope, clientValues.generateUsers),
      generateEnvironments = normalizeGlobalScope(scope, clientValues.generateEnvironments),
      generateApplications = normalizeGlobalScope(scope, clientValues.generateApplications),
      includeSecrets = clientValues.includeSecrets,
      generateTemplates = needTemplates || normalizeAnyScope(scope, clientValues.generateTemplates, needAllFolderTypes),
      generateWorkflows = needTemplates || normalizeAnyScope(scope, clientValues.generateWorkflows, needAllFolderTypes),
      generateTriggers = normalizeFolderScope(scope, clientValues.generateTriggers, needAllFolderTypes),
      generateDeliveryPatterns = normalizeFolderScope(scope, clientValues.generateDeliveryPatterns, needAllFolderTypes),
      generateDashboards = normalizeAnyScope(scope, clientValues.generateDashboards, needAllFolderTypes),
      generateConfigurations = normalizeAnyScope(scope, clientValues.generateConfigurations, needAllFolderTypes),
      generateSettings = normalizeGlobalScope(scope, clientValues.generateSettings),
      generateRiskProfiles = normalizeGlobalScope(scope, clientValues.generateRiskProfiles),
      generateVariables = normalizeAnyScope(scope, clientValues.generateVariables, needAllFolderTypes),
      generateNotificationSettings = normalizeAnyScope(scope, clientValues.generateNotificationSettings, needAllFolderTypes),
      generateCalendar = normalizeGlobalScope(scope, clientValues.generateCalendar),
      generateFolderVersioning = normalizeFolderScope(scope, clientValues.generateFolderVersioning, needAllFolderTypes)
    )
  }

}

==> xlr-devops-as-code-module-23.3.0-713.113-sources.jar: com/xebialabs/xlrelease/ascode/rest/XLRAsCodeResource.scala <==
package com.xebialabs.xlrelease.ascode.rest

import com.xebialabs.ascode.annotation.ExceptionToAsCodeResponse
import com.xebialabs.ascode.exception.AsCodeException
import com.xebialabs.ascode.yaml.model.Definition
import com.xebialabs.ascode.yaml.writer.DefinitionWriter.WriterConfig
import com.xebialabs.deployit.booter.local.utils.Strings
import com.xebialabs.deployit.core.rest.resteasy.Workdir
import com.xebialabs.deployit.core.rest.resteasy.Workdir.Clean.ONLY_ON_EXCEPTION
import com.xebialabs.deployit.security.permission.PlatformPermissions.ADMIN
import com.xebialabs.xlplatform.coc.dto.SCMTraceabilityData
import com.xebialabs.xlrelease.ascode.schema.JsonSchemaGenerator
import com.xebialabs.xlrelease.ascode.service.GenerateService.{CisConfig, GeneratorConfig}
import com.xebialabs.xlrelease.ascode.service._
import com.xebialabs.xlrelease.ascode.service.previewhandler.PreviewService
import com.xebialabs.xlrelease.ascode.utils.StaticVariables
import com.xebialabs.xlrelease.repository.Ids
import com.xebialabs.xlrelease.repository.Ids.SEPARATOR
import com.xebialabs.xlrelease.security.PermissionChecker
import grizzled.slf4j.Logging
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Controller

import javax.ws.rs._
import javax.ws.rs.core.Response

@Controller
@ExceptionToAsCodeResponse
@Path("/devops-as-code")
class XLRAsCodeResource @Autowired()(creationService: CreationService,
                                     generateService: GenerateService,
                                     previewService: PreviewService,
                                     jsonGenerator: JsonSchemaGenerator,
                                     permissionChecker: PermissionChecker) extends Logging {
  @POST
  @Produces(Array("application/json"))
  @Path("/apply")
  def createPipeline(definition: Definition, scmData: SCMTraceabilityData): Response = {
    try {
      val response = creationService.processBlueprint(definition, scmData)
      Response.ok(response).build()
    } catch {
      case e: Throwable =>
        logger.error(e.getMessage, e)
        throw e
    }
  }

  //noinspection ScalaStyle
  @GET
  @Produces(Array("application/zip", "text/vnd.yaml"))
  @Workdir(prefix = StaticVariables.AS_CODE_GENERATE_PREFIX, clean = ONLY_ON_EXCEPTION)
  @Path("/generate")
  def generateCi(@QueryParam("path") path: String,
                 @QueryParam("ciName") ciName: String,
                 @QueryParam("ciIds") ciIds: Array[String],
                 @DefaultValue("false") @QueryParam("defaults") defaults: Boolean,
                 @DefaultValue("false") @QueryParam("permissions") permissions: Boolean,
                 @DefaultValue("false") @QueryParam("users") users: Boolean,
                 @DefaultValue("false") @QueryParam("roles") roles: Boolean,
                 @DefaultValue("false") @QueryParam("environments") environments: Boolean,
                 @DefaultValue("false") @QueryParam("applications") applications: Boolean,
                 @DefaultValue("false") @QueryParam("secrets") includeSecrets: Boolean,
                 @DefaultValue("false") @QueryParam("templates") templates: Boolean,
                 @DefaultValue("false") @QueryParam("triggers") triggers: Boolean,
                 @DefaultValue("false") @QueryParam("dashboards") dashboards: Boolean,
                 @DefaultValue("false") @QueryParam("configurations") configurations: Boolean,
                 @DefaultValue("false") @QueryParam("settings") settings: Boolean,
                 @DefaultValue("false") @QueryParam("riskProfiles") riskProfiles: Boolean,
                 @DefaultValue("false") @QueryParam("deliveryPatterns") deliveryPatterns: Boolean,
                 @DefaultValue("false") @QueryParam("variables") variables: Boolean,
                 @DefaultValue("false") @QueryParam("notifications") notificationSettings: Boolean,
                 @DefaultValue("false") @QueryParam("calendar") calendar: Boolean,
                 @DefaultValue("true") @QueryParam("includeSubFolders") includeSubFolders: Boolean,
                 @DefaultValue("false") @QueryParam("folderVersioning") folderVersioning: Boolean): Response = {

    // Only templates support generate by id
    if (!ciIds.forall(Ids.isReleaseId)) {
      throw new AsCodeException("Only template ids can be specified")
    }

    val searchScope = if (!ciIds.isEmpty) {
      ExactSearch(ciIds.toList)
    } else if (Strings.isNotBlank(path)) {
      if (path == SEPARATOR) {
        GlobalSearch
      } else {
        FolderSearch(path, null, includeSubFolders)
      }
    } else {
      AllSearch
    }

    val cisConfig = normalizeCisConfig(searchScope,
      CisConfig(
        permissions,
        roles,
        users,
        environments,
        applications,
        includeSecrets,
        templates,
        triggers,
        deliveryPatterns,
        dashboards,
        configurations,
        settings,
        riskProfiles,
        variables,
        notificationSettings,
        calendar,
        folderVersioning
      )
    )

    try {
      val generatorConfig = GeneratorConfig(Option(ciName).filter(_.nonEmpty), searchScope, cisConfig, permissionChecker.hasGlobalPermission(ADMIN))
      val definitions = generateService.generate(generatorConfig)
      Response.ok(WriterConfig(definitions, includeSecrets, writeDefaults = defaults)).build()
    } catch {
      case e: Throwable =>
        logger.error(e.getMessage, e)
        throw e
    }
  }

  @POST
  @Path("/preview")
  @Workdir(prefix = StaticVariables.AS_CODE_GENERATE_PREFIX, clean = ONLY_ON_EXCEPTION)
  @Produces(Array("application/json"))
  def preview(definition: Definition): Response = {
    val response = previewService.preview(definition)
    Response.ok(response).build()
  }

  @GET
  @Path("/schema")
  @Produces(Array("application/json"))
  def schema(): Response = {
    Response.ok(jsonGenerator.YamlSchema).build()
  }

  private def normalizeFolderScope(scope: SearchScope, clientValue: Boolean, folderScopeDefault: Boolean): Boolean = {
    scope match {
      case AllSearch => clientValue
      case GlobalSearch => false
      case _: FolderSearch => clientValue | folderScopeDefault
      case _: ExactSearch => false
    }
  }

  private def normalizeGlobalScope(scope: SearchScope, clientValue: Boolean): Boolean = {
    scope match {
      case AllSearch => clientValue
      case GlobalSearch => clientValue
      case _: FolderSearch => false
      case _: ExactSearch => false
    }
  }

  private def normalizeAnyScope(scope: SearchScope, clientValue: Boolean, folderScopeDefault: Boolean): Boolean = {
    scope match {
      case AllSearch => clientValue
      case GlobalSearch => clientValue
      case _: FolderSearch => clientValue | folderScopeDefault
      case _: ExactSearch => false
    }
  }

  private def validateScopeAndInput(scope: SearchScope, clientValues: CisConfig): Unit = {
    scope match {
      case GlobalSearch if !clientValues.hasGlobalTypes() && clientValues.hasFolderTypes() =>
        throw new AsCodeException("Could not generate a definition for path '/' when only folder-scoped types are requested")
      case FolderSearch(path, _, _) if clientValues.hasGlobalTypes() && !clientValues.hasFolderTypes() =>
        throw new AsCodeException(s"Could not generate a definition for folder '$path' when only global-scoped types are requested")
      case AllSearch if clientValues.isEmpty() =>
        throw new AsCodeException("Could not generate a definition without a path or some types specified")
      case _ =>
    }
  }

  private def normalizeCisConfig(scope: SearchScope, clientValues: CisConfig): CisConfig = {
    validateScopeAndInput(scope, clientValues)

    // When folder scope is needed but all folder types excluded it should default to all folder types
    val needAllFolderTypes = scope match {
      case _: FolderSearch => !clientValues.hasFolderTypes()
      case _ => false
    }

    val needTemplates = scope.isInstanceOf[ExactSearch]

    // Normalize so all values are only enabled when they are relevant to the scope being generated
    CisConfig(
      normalizeAnyScope(scope, clientValues.generatePermissions, needAllFolderTypes),
      normalizeGlobalScope(scope, clientValues.generateRoles),
      normalizeGlobalScope(scope, clientValues.generateUsers),
      normalizeGlobalScope(scope, clientValues.generateEnvironments),
      normalizeGlobalScope(scope, clientValues.generateApplications),
      clientValues.includeSecrets,
      needTemplates || normalizeAnyScope(scope, clientValues.generateTemplates, needAllFolderTypes),
      normalizeFolderScope(scope, clientValues.generateTriggers, needAllFolderTypes),
      normalizeFolderScope(scope, clientValues.generateDeliveryPatterns, needAllFolderTypes),
      normalizeAnyScope(scope, clientValues.generateDashboards, needAllFolderTypes),
      normalizeAnyScope(scope, clientValues.generateConfigurations, needAllFolderTypes),
      normalizeGlobalScope(scope, clientValues.generateSettings),
      normalizeGlobalScope(scope, clientValues.generateRiskProfiles),
      normalizeAnyScope(scope, clientValues.generateVariables, needAllFolderTypes),
      normalizeAnyScope(scope, clientValues.generateNotificationSettings, needAllFolderTypes),
      normalizeGlobalScope(scope, clientValues.generateCalendar),
      normalizeFolderScope(scope, clientValues.generateFolderVersioning, needAllFolderTypes)
    )
  }

}
